#include "chunk.h"

void chunkLoad(chunklist_t *list, chunk_t *chunk,
  int32_t x, int32_t y, int32_t z
) {
  int32_t tx, ty, tz, i;
  tile_t *tile;
  tiledef_t *tileDef;
  tilesetdiv_t *div;

  //TODO: Actually load here, this should be combined with below
  for(i = 0; i < list->tileCount; i++) {
    (chunk->tiles + i)->id = (i%2) + 1;
  }
  
  // Determine the size of our primitive.
  int32_t indiceCount = 0, verticeCount = 0;
  for(i = 0; i < list->tileCount; i++) {
    tile = chunk->tiles + i;
    if(tile->id == TILE_NULL) continue;
    tileDef = list->tilemap->tileDefinitions + tile->id;
    if(tileDef->verticeCount == 0 || tileDef->indiceCount == 0) continue;

    tile->verticeStart = verticeCount;
    tile->indiceStart = indiceCount;

    verticeCount += tileDef->verticeCount;
    indiceCount += tileDef->indiceCount;
  }

  // Now we know how big the primitive needs to be!
  chunk->primitive = primitiveCreate(verticeCount, indiceCount);

  // For each tile
  i = 0;
  for(tx = 0; tx < CHUNK_WIDTH; tx++) {
    for(ty = 0; ty < CHUNK_HEIGHT; ty++) {
      for(tz = 0; tz < CHUNK_DEPTH; tz++) {
        // Get tile for position...
        tile = chunk->tiles + (i++);

        // Should Chunk bother rendering?
        if(tile->id == TILE_NULL) continue;
        tileDef = list->tilemap->tileDefinitions + tile->id;
        if(tileDef->verticeCount == 0 || tileDef->indiceCount == 0) continue;

        // Render the tile's primitive
        tileRender(list, chunk, tile, tileDef, tx, ty, tz);
      }
    }
  }
}

void chunkUnload(chunklist_t *list, chunk_t *chunk) {
  // Unload the primitive
  primitiveDispose(chunk->primitive);
  chunk->primitive = NULL;

  // Load chunks to zero. TODO: Necessary?
  memset(chunk->tiles, TILE_NULL, list->count);
}