// Copyright (c) 2021 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include <stdint.h>
#include <malloc.h>
#include "defs.h"
#include "../display/tileset.h"
#include "../display/primitives/quad.h"
#include "../display/primitives/cube.h"

#define TILE_NULL 0
#define TILE_FLAG_DYNAMIC (tileflag_t)1

/**
 * Creates a tilemap from a tileset.
 * 
 * @param tileset Tileset to create from.
 * @returns The tilemap.
 */
tilemap_t * tileMapCreate(tileset_t *tileset);

/**
 * Destroys a previously created tilemap.
 * 
 * @param tilemap Tilemap to dispose.
 */
void tileMapDispose(tilemap_t *tilemap);

/**
 * Rendedrs a given chunk tile.
 * 
 * @param list The chunk list that the chunk belongs to.
 * @param chunk The chunk to render against.
 * @param tile The tile to render.
 * @param tileDef The tile's definition information.
 * @param x The X Coordinate of the tile (chunk space).
 * @param y The Y Coordinate of the tile (chunk space).
 * @param z The Z Coordinate of the tile (chunk space).
 */
void tileRender(chunklist_t *list, chunk_t *chunk,
  tile_t *tile, tiledef_t *tileDef, int32_t x, int32_t y, int32_t z
);