/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "world.h"

world_t * worldCreate() {
  world_t *world = malloc(sizeof(world_t));
  if(world == NULL) return NULL;

  // Texture
  world->texture = assetTextureLoad("tileset.png");
  if(world->texture == NULL) {
    free(world);
    return NULL;
  }

  // Tileset
  world->tileset = tilesetCreate(
    world->texture->width/16, world->texture->height/16,
    world->texture->width,    world->texture->height,
    0, 0, 0, 0
  );
  if(world->tileset == NULL) {
    textureDispose(world->texture);
    free(world);
    return NULL;
  }

  // Tilemap
  world->tilemap = tileMapCreate(world->tileset);
  if(world->tilemap == NULL) {
    textureDispose(world->texture);
    tilesetDispose(world->tileset);
    free(world);
    return NULL;
  }

  // Chunk Lists
  world->chunkList = chunkListCreate(1, 1, 1, world->tilemap);
  if(world->chunkList == NULL) {
    textureDispose(world->texture);
    tilesetDispose(world->tileset);
    tileMapDispose(world->tilemap);
    free(world);
    return NULL;
  }

  return world;
}

void worldRender(world_t *world, shader_t *shader) {
  int32_t i;

  shaderUseTexture(shader, world->texture);
  chunkListRender(world->chunkList, shader);
}

void worldDispose(world_t *world) {
  textureDispose(world->texture);
  tilesetDispose(world->tileset);
  tileMapDispose(world->tilemap);
  chunkListDispose(world->chunkList);
  free(world);
}